/*
   This file is part of the Aulib library.
   Name: amdf.cpp
   Copyright (C) 2004 Silvia Pfeiffer

   Lehrstuhl fuer Praktische Informatik IV
   Universitaet Mannheim
   68131 Mannheim
   Germany
   email: pfeiffer@pi4.informatik.uni-mannheim.de

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/

#include "amdf.h"
#include "aucache.h"
#include <cstdlib>

using namespace std;

double AuAmdf::amdf(const AudioCache& cache, int delay) {
  int sum = 0;

  for (size_t i = delay; i < cache.size(); i++) {
    sum += abs(cache[i] - cache[i-delay]);
  }

  return (double)sum/(double)cache.samplingRate();
}

int AuAmdf::amdfMax(const AudioCache& cache) {
  double maxsum = 0;
  int delay = 0;
  double amd=0;

  for (size_t i = 1; i < cache.size(); i++) {
    amd = amdf(cache,i);
    if (amd > maxsum) {
      maxsum = amd;
      delay = i;
    }
  }

  return delay;
}

int AuAmdf::amdfMin(const AudioCache& cache) {
  double minsum = FLT_MAX;
  int delay = 0;
  double amd=0;

  for (size_t i = 1; i < cache.size(); i++) {
    amd = amdf(cache,i);
    if (amd < minsum) {
      minsum = amd;
      delay = i;
    }
  }

  return delay;
}
