/*
   This file is part of the Aulib library.
   Name: autocorr.h
   Copyright (C) 2004 Silvia Pfeiffer

   Lehrstuhl fuer Praktische Informatik IV
   Universitaet Mannheim
   68131 Mannheim
   Germany
   email: pfeiffer@pi4.informatik.uni-mannheim.de

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/

#ifndef _AU_AUTOCORR_H_
#define _AU_AUTOCORR_H_

#include "autypes.h"

#include <cmath>
#include <climits>
#include <cfloat>

using namespace std;

class AudioCache;

/**
 * This class implements routines to calculate the autocorrelation
 * of the samples stored in the audio cache in the time domain.
 * This may either be done for a certain time delay or for all possible
 * time delays.
 *
 * @author <a href = "mailto:pfeiffer@pi4.informatik.uni-mannheim.de">Silvia Pfeiffer</a>
 * @version $Id: autocorr.h,v 1.1 1998/06/04 09:30:58 pfeiffer Exp $
 */
class AuAutocorr {

  public:
    /**
      * Function to calculate autocorrelation on the samples stored in the
      * audio cache. The original definition of the auto correlation of a
      * discrete-time signal is:<BR>
      * r(e) = lim{n->infty} 1/sampling_rate *
      *        sum{-n to n} (s(n) * s(n-e))
      *
      * @param cache audio cache containing the signal
      * @param delay time delay
      * @return      calculated auto correlation
      */
    static double autocorr(const AudioCache& cache, const int delay);

    /**
      * Function to calculate the time delay which gives a maximal
      * autocorrelation on the samples stored in the audio cache.
      *
      * @param cache audio cache containing the signal
      * @return      calculated auto correlation
      */
    static int autocorrCache(const AudioCache& cache);
};

#endif

