/*
   This file is part of the Aulib library.
   Name: error.h
   Copyright (C) 2004 Silvia Pfeiffer

   Lehrstuhl fuer Praktische Informatik IV
   Universitaet Mannheim
   68131 Mannheim
   Germany
   email: pfeiffer@pi4.informatik.uni-mannheim.de

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/

#ifndef _EXCEPTION_H_
#define _EXCEPTION_H_

#include <string>
using namespace std;

/**
 * This class encapsulates information about an error. It should be
 * created and thrown when an error occurres.<BR>
 * <TT>Exception</TT> was implemented to be the base class for
 * a set of exception classes, but GNUs g++ does not allow to
 * use inheritance by try-throw-catch. So do not make subclasses
 * of this class and use only this for exceptions.
 *
 * @author <a href = "mailto:pfeiffer@pi4.informatik.uni-mannheim.de">Silvia Pfeiffer</a>, <a href = "mailto:tomczyk@chili.wifo.uni-mannheim.de">Peter Tomczyk</a>
 * @version $Id: error.h,v 1.8 1998/09/11 11:18:46 tomczyk Exp $
 */
class Exception {
  
  public:

    /**
     * creates new exception instance.
     * @param s string describing the error
     */
    Exception(const string& s) : message(s) { }

    /**
     * returns the error description.
     * @return descrition
     */
    const string& what() const { return message; }

  protected:

    /// error description.
    string message;
};

#endif
