/*
   This file is part of the Aulib library.
   Name: rangeTableFile.cpp
   Copyright (C) 2004 Silvia Pfeiffer

   Lehrstuhl fuer Praktische Informatik IV
   Universitaet Mannheim
   68131 Mannheim
   Germany
   email: pfeiffer@pi4.informatik.uni-mannheim.de

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/

#include <iostream.h>
#include <fstream.h>
#include <cstring>
#include <cstdlib>
#include <cstdio>
#include "byteOrder.h"
#include "scalarFile.H"
#include "rangeTableFile.H"

#ifndef _WIN32
#ifdef DAMN_INTEL_BYTE_ORDER
//#include <sys/mode.h>
#endif

#include <sys/types.h>
#include <sys/stat.h>
#endif not _WIN32

// ---------------------------------------------------------------------
// -  CLASS rangeTableFile
// ---------------------------------------------------------------------
int
rangeTableFile::newDB(string _filename,
		      int _startFrameNo, int _stopFrameNo)
{
  filename = _filename;
  startFrameNo = _startFrameNo;
  stopFrameNo = _stopFrameNo;

  ofstream file(filename.c_str());

  if ( !file) {
    cerr << "Cannot create DB file "
	 << filename.c_str()
	 << "." << endl;
    return false;
  }

  // write header
  file << startFrameNo
       << "\t"
       << stopFrameNo
       << endl;
  file.close();
#ifndef _WIN32
  chmod (filename.c_str(),S_IRUSR|S_IWUSR|S_IRGRP|S_IWGRP);
#endif not _WIN32

  return true;
}


int
rangeTableFile::openDB(string _filename,
		       int _startFrameNo, int _stopFrameNo)
{
  filename = _filename;
  startFrameNo = _startFrameNo;
  stopFrameNo = _stopFrameNo;

  ifstream file(filename.c_str());
  if (!file) {
    return newDB(filename, startFrameNo, stopFrameNo);
  }

  // check, whether DB file has write size
  int dbstartFrameNo, dbstopFrameNo;
  file >> dbstartFrameNo
       >> dbstopFrameNo;
  file.close();

  if ( (dbstartFrameNo != startFrameNo) ||
       (dbstopFrameNo != stopFrameNo) ) {
    // DB has wrong size, generate new
    return newDB(filename, startFrameNo, stopFrameNo);
  }

  return true;
}


int
rangeTableFile::save(rangeTableType value)
{
  ofstream file(filename.c_str());
  if ( !file) {
    cerr << "Cannot create DB file "
	 << filename.c_str()
	 << "." << endl;
    return false;
  }

  // write header
  file << startFrameNo
       << "\t"
       << stopFrameNo
       << endl
       << value;
  file.close();

  return true;
}


rangeTableType
rangeTableFile::load(int _startFrameNo, int _stopFrameNo)
{
  rangeTableType value;

  ifstream file(filename.c_str());
  if (!file) return value;

  // check, whether DB file has write size
  int dbstartFrameNo, dbstopFrameNo;
  file >> dbstartFrameNo
       >> dbstopFrameNo
       >> value;
  file.close();

  return value;
}

rangeTableFile::rangeTableFile () {startFrameNo=-1; stopFrameNo=-1;}
rangeTableFile::~rangeTableFile () {}
