/*
   This file is part of the Aulib library.
   Name: rangeTableType.H
   CCopyright (C) 2004 Silvia Pfeiffer

   Lehrstuhl fuer Praktische Informatik IV
   Universitaet Mannheim
   68131 Mannheim
   Germany
   email: pfeiffer@pi4.informatik.uni-mannheim.de

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/

#ifndef _RANGETABLETYPE_H
#define _RANGETABLETYPE_H


#ifndef _WIN32
#include <string>
#else
#define __MSCPLUSPLUS__
#include <Bstring.h>
#endif
#include <iostream.h>
#include <vector>

// used for tml_resolution, tml_rangeTable
struct RangeTableEntryType {
  RangeTableEntryType() : start(-1), stop(-1), text("") {}
  RangeTableEntryType(string s , int i1, int i2) : text(s), start(i1), stop(i2) {};
  RangeTableEntryType(const RangeTableEntryType& rte) {
    text = rte.text;
    start = rte.start;
    stop = rte.stop;
  }
  
  RangeTableEntryType& operator=(const RangeTableEntryType& rte) {
    if (this == &rte) 
      return *this;
    text = rte.text;
    start = rte.start;
    stop = rte.stop;
    return *this;
  }

  int noOfCoveredFrames() const { return stop-start+1; }

  // relations with respect to start frame number
  friend bool operator< (const RangeTableEntryType& rte1, const RangeTableEntryType& rte2) {
    return (rte1.start < rte2.start);
  }

  string text;
  int start, stop;
};



class rangeTableType {
public:
  friend ostream& operator<< (ostream&, const rangeTableType&);
  friend istream& operator>>(istream&, rangeTableType&);

  rangeTableType() {};
  rangeTableType(const rangeTableType& rt);
  rangeTableType& operator= (const rangeTableType& rt);
  ~rangeTableType() {}

  // index operator
  RangeTableEntryType& operator[](int i) {
    if ((i>=0) && (i<rt.size()))
      return rt[i];
    cerr << "Range Error in rangeTableType" << endl;
    return rt[0];
  }

  // converts string into rangeTableType
  void fromString (string s);

  // converts rangeTableType into string
  string rangeTableType::toString ();

  // return range the frameNo falls into
  RangeTableEntryType getRange(int frameNo);
  // return new rangeTable which holds only entries of the specified range.
  rangeTableType getRange(int startFrameNo, int stopFrameNo);
  rangeTableType getRange(const RangeTableEntryType& rte);
  bool delRange(int frameNo);
  bool delRange(int startFrameNo, int stopFrameNo);
  bool delRange(string text); //included by S.P.
  bool delRange(const RangeTableEntryType& rte);

  // calculate total number of frames covered by the rangeTable
  int noOfCoveredFrames() const;
  void add(RangeTableEntryType& rte) {rt.push_back(rte);}
  void add(rangeTableType& rt);

  size_t fileSize() {return rt.size();}
  size_t size() {return rt.size();}
  vector<RangeTableEntryType>::iterator begin() { return rt.begin(); }
  vector<RangeTableEntryType>::iterator end() { return rt.end(); }

//private:
  vector<RangeTableEntryType> rt;
};

#endif _RANGETABLETYPE_H

