/*
   This file is part of the Aulib library.
   Name: rangeTableVectorType.H
   Copyright (C) 2004 Silvia Pfeiffer

   Lehrstuhl fuer Praktische Informatik IV
   Universitaet Mannheim
   68131 Mannheim
   Germany
   email: pfeiffer@pi4.informatik.uni-mannheim.de

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/

#ifndef _RANGETABLEVECTORTYPE_H
#define _RANGETABLEVECTORTYPE_H


#ifndef _WIN32
#include <string>
#else
#define __MSCPLUSPLUS__
#include <Bstring.h>
#endif
#include <iostream.h>
#include <vector>

struct RangeTableVEType {
  RangeTableVEType() : start(-1), stop(-1), text(""), offset(-1) {}
  RangeTableVEType(string s , int i1, int i2, long o) : text(s), start(i1), stop(i2), offset(o) {};
  RangeTableVEType(const RangeTableVEType& rte) {
    text = rte.text;
    start = rte.start;
    stop = rte.stop;
    offset = rte.offset;
  }

  RangeTableVEType& operator=(const RangeTableVEType& rte) {
    if (this == &rte)
      return *this;
    text = rte.text;
    start = rte.start;
    stop = rte.stop;
    offset = rte.offset;
    return *this;
  }

  int noOfCoveredFrames() const { return stop-start+1; }

  // relations with respect to start frame number
  friend bool operator< (const RangeTableVEType& rte1, const RangeTableVEType& rte2) {
    return (rte1.start < rte2.start);
  }

  string text;
  int start, stop;
  long offset;
};



class rangeTableVectorType {
public:
  friend ostream& operator<< (ostream&, const rangeTableVectorType&);
  friend istream& operator>>(istream&, rangeTableVectorType&);

  rangeTableVectorType() {};
  rangeTableVectorType(const rangeTableVectorType& rt);
  rangeTableVectorType& operator= (const rangeTableVectorType& rt);
  ~rangeTableVectorType() {}

  // index operator
  RangeTableVEType& operator[](int i) {
    if ((i>=0) && (i<rt.size()))
      return rt[i];
    cerr << "Range Error in rangeTableVectorType" << endl;
    return rt[0];
  }

  // converts string into rangeTableVectorType
  void fromString (string s);

  // converts rangeTableVectorType into string
  string rangeTableVectorType::toString ();

  // return range the frameNo falls into
  RangeTableVEType getRange(int frameNo);
  // return new rangeTable which holds only entries of the specified range.
  rangeTableVectorType getRange(int startFrameNo, int stopFrameNo);
  rangeTableVectorType getRange(const RangeTableVEType& rte);
  bool delRange(int frameNo);
  bool delRange(int startFrameNo, int stopFrameNo);
  bool delRange(string text);
  bool delRange(const RangeTableVEType& rte);

  // calculate total number of frames covered by the rangeTable
  int noOfCoveredFrames() const;
  void add(RangeTableVEType& rte);
  void add(rangeTableVectorType& rt);

  size_t fileSize() {return rt.size();}
  size_t size() {return rt.size();}
  vector<RangeTableVEType>::iterator begin() { return rt.begin(); }
  vector<RangeTableVEType>::iterator end() { return rt.end(); }

//private:
  vector<RangeTableVEType> rt;
};

#endif _RANGETABLEVECTORTYPE_H

